@@ -53,13 +53,13 @@ import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
 import java.util.concurrent.ExecutionException;
-import java.util.concurrent.ExecutorService;
-import java.util.concurrent.TimeUnit;
+import java.util.concurrent.Executor;
 import java.util.concurrent.atomic.AtomicBoolean;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.api.MailAccess;
 import com.openexchange.mail.dataobjects.MailFolder;
 import com.openexchange.mail.dataobjects.ReadOnlyMailFolder;
+import com.openexchange.mail.dataobjects.MailFolder.DefaultFolderType;
 import com.openexchange.mail.permission.DefaultMailPermission;
 import com.openexchange.mail.permission.MailPermission;
 import com.openexchange.mailaccount.MailAccount;
@@ -69,12 +69,14 @@ import com.openexchange.mailaccount.UnifiedINBOXManagement;
 import com.openexchange.server.ServiceException;
 import com.openexchange.server.impl.OCLPermission;
 import com.openexchange.session.Session;
+import com.openexchange.threadpool.ThreadPoolService;
+import com.openexchange.threadpool.ThreadPools;
+import com.openexchange.unifiedinbox.UnifiedINBOXAccess;
 import com.openexchange.unifiedinbox.UnifiedINBOXException;
 import com.openexchange.unifiedinbox.services.UnifiedINBOXServiceRegistry;
 import com.openexchange.unifiedinbox.utility.LoggingCallable;
 import com.openexchange.unifiedinbox.utility.TrackingCompletionService;
 import com.openexchange.unifiedinbox.utility.UnifiedINBOXCompletionService;
-import com.openexchange.unifiedinbox.utility.UnifiedINBOXExecutors;
 import com.openexchange.unifiedinbox.utility.UnifiedINBOXUtility;
 
 /**
@@ -86,7 +88,8 @@ public final class UnifiedINBOXFolderConverter {
 
     static final int[] EMPTY_COUNTS = new int[] { 0, 0, 0, 0 };
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(UnifiedINBOXFolderConverter.class);
+    private static final org.apache.commons.logging.Log LOG =
+        org.apache.commons.logging.LogFactory.getLog(UnifiedINBOXFolderConverter.class);
 
     private static final MailFolder ROOT_UNIFIED_INBOX_FOLDER;
 
@@ -122,6 +125,7 @@ public final class UnifiedINBOXFolderConverter {
             tmp.addPermission(permission);
         }
         tmp.setDefaultFolder(false);
+        tmp.setDefaultFolderType(DefaultFolderType.NONE);
         tmp.setMessageCount(-1);
         tmp.setNewMessageCount(-1);
         tmp.setUnreadMessageCount(-1);
@@ -170,7 +174,7 @@ public final class UnifiedINBOXFolderConverter {
      * @return The appropriately filled instance of {@link MailFolder}
      * @throws MailException If converting mail folder fails
      */
-    public static MailFolder getUnifiedINBOXFolder(final int unifiedInboxAccountId, final Session session, final String fullname, final String localizedName, final ExecutorService executor) throws MailException {
+    public static MailFolder getUnifiedINBOXFolder(final int unifiedInboxAccountId, final Session session, final String fullname, final String localizedName, final Executor executor) throws MailException {
         final MailFolder tmp = new MailFolder();
         // Subscription not supported by Unified INBOX, so every folder is "subscribed"
         tmp.setSubscribed(true);
@@ -200,6 +204,19 @@ public final class UnifiedINBOXFolderConverter {
         }
         // What else?!
         tmp.setDefaultFolder(true);
+        if (UnifiedINBOXAccess.INBOX.equals(fullname)) {
+            tmp.setDefaultFolderType(DefaultFolderType.INBOX);
+        } else if (UnifiedINBOXAccess.TRASH.equals(fullname)) {
+            tmp.setDefaultFolderType(DefaultFolderType.TRASH);
+        } else if (UnifiedINBOXAccess.SENT.equals(fullname)) {
+            tmp.setDefaultFolderType(DefaultFolderType.SENT);
+        } else if (UnifiedINBOXAccess.SPAM.equals(fullname)) {
+            tmp.setDefaultFolderType(DefaultFolderType.SPAM);
+        } else if (UnifiedINBOXAccess.DRAFTS.equals(fullname)) {
+            tmp.setDefaultFolderType(DefaultFolderType.DRAFTS);
+        } else {
+            tmp.setDefaultFolderType(DefaultFolderType.NONE);
+        }
         // Set message counts
         final boolean hasAtLeastOneSuchFolder = setMessageCounts(fullname, unifiedInboxAccountId, session, tmp, executor);
         if (hasAtLeastOneSuchFolder) {
@@ -209,12 +226,11 @@ public final class UnifiedINBOXFolderConverter {
         return tmp;
     }
 
-    private static boolean setMessageCounts(final String fullname, final int unifiedInboxAccountId, final Session session, final MailFolder tmp, final ExecutorService executor) throws UnifiedINBOXException, MailException {
+    private static boolean setMessageCounts(final String fullname, final int unifiedInboxAccountId, final Session session, final MailFolder tmp, final Executor executor) throws UnifiedINBOXException, MailException {
         final MailAccount[] accounts;
         try {
-            final MailAccountStorageService storageService = UnifiedINBOXServiceRegistry.getServiceRegistry().getService(
-                MailAccountStorageService.class,
-                true);
+            final MailAccountStorageService storageService =
+                UnifiedINBOXServiceRegistry.getServiceRegistry().getService(MailAccountStorageService.class, true);
             final MailAccount[] arr = storageService.getUserMailAccounts(session.getUserId(), session.getContextId());
             final List<MailAccount> l = new ArrayList<MailAccount>(arr.length);
             for (int i = 0; i < arr.length; i++) {
@@ -231,14 +247,11 @@ public final class UnifiedINBOXFolderConverter {
         }
         // Create completion service for simultaneous access
         final int length = accounts.length;
-        final ExecutorService exec;
-        final boolean shutdown;
+        final Executor exec;
         if (executor == null) {
-            exec = UnifiedINBOXExecutors.newCachedThreadPool(length);
-            shutdown = true;
+            exec = UnifiedINBOXServiceRegistry.getServiceRegistry().getService(ThreadPoolService.class).getExecutor();
         } else {
             exec = executor;
-            shutdown = false;
         }
         final TrackingCompletionService<int[]> completionService = new UnifiedINBOXCompletionService<int[]>(exec);
         final AtomicBoolean retval = new AtomicBoolean();
@@ -266,9 +279,10 @@ public final class UnifiedINBOXFolderConverter {
                             }
                             // Get counts
                             final MailFolder mailFolder = mailAccess.getFolderStorage().getFolder(accountFullname);
-                            final int[] counts = new int[] {
-                                mailFolder.getMessageCount(), mailFolder.getUnreadMessageCount(), mailFolder.getDeletedMessageCount(),
-                                mailFolder.getNewMessageCount() };
+                            final int[] counts =
+                                new int[] {
+                                    mailFolder.getMessageCount(), mailFolder.getUnreadMessageCount(), mailFolder.getDeletedMessageCount(),
+                                    mailFolder.getNewMessageCount() };
                             retval.set(true);
                             return counts;
                         } finally {
@@ -308,25 +322,7 @@ public final class UnifiedINBOXFolderConverter {
             Thread.currentThread().interrupt();
             throw new MailException(MailException.Code.INTERRUPT_ERROR, e);
         } catch (final ExecutionException e) {
-            final Throwable t = e.getCause();
-            if (MailException.class.isAssignableFrom(t.getClass())) {
-                throw (MailException) t;
-            } else if (t instanceof RuntimeException) {
-                throw (RuntimeException) t;
-            } else if (t instanceof Error) {
-                throw (Error) t;
-            } else {
-                throw new IllegalStateException("Not unchecked", t);
-            }
-        } finally {
-            if (shutdown) {
-                try {
-                    exec.shutdownNow();
-                    exec.awaitTermination(10000L, TimeUnit.MILLISECONDS);
-                } catch (final InterruptedException e) {
-                    Thread.currentThread().interrupt();
-                }
-            }
+            throw ThreadPools.launderThrowable(e, MailException.class);
         }
     }
 
@@ -368,8 +364,8 @@ public final class UnifiedINBOXFolderConverter {
                 final String accountFullname = UnifiedINBOXUtility.determineAccountFullname(mailAccess, fullnames[i]);
                 if (null != accountFullname && mailAccess.getFolderStorage().exists(accountFullname)) {
                     final MailFolder mf = mailAccess.getFolderStorage().getFolder(accountFullname);
-                    retval[i] = new int[] {
-                        mf.getMessageCount(), mf.getUnreadMessageCount(), mf.getDeletedMessageCount(), mf.getNewMessageCount() };
+                    retval[i] =
+                        new int[] { mf.getMessageCount(), mf.getUnreadMessageCount(), mf.getDeletedMessageCount(), mf.getNewMessageCount() };
                 } else if (LOG.isDebugEnabled()) {
                     LOG.debug(new StringBuilder(32).append("Missing folder \"").append(fullnames[i]).append("\" in account ").append(
                         accountId).toString());
@@ -398,12 +394,13 @@ public final class UnifiedINBOXFolderConverter {
         for (int i = 0; i < retval.length; i++) {
             final MailFolder tmp = retval[i] = new MailFolder();
             // Subscription not supported by Unified INBOX, so every folder is "subscribed"
+            final String fullname = fullnames[i];
             tmp.setSubscribed(true);
             tmp.setSupportsUserFlags(true);
             tmp.setRootFolder(false);
             tmp.setExists(true);
             tmp.setSeparator('/');
-            tmp.setFullname(fullnames[i]);
+            tmp.setFullname(fullname);
             tmp.setParentFullname(MailFolder.DEFAULT_FOLDER_ID);
             tmp.setName(localizedNames[i]);
             tmp.setHoldsFolders(true);
@@ -416,6 +413,19 @@ public final class UnifiedINBOXFolderConverter {
             }
             // What else?!
             tmp.setDefaultFolder(true);
+            if (UnifiedINBOXAccess.INBOX.equals(fullname)) {
+                tmp.setDefaultFolderType(DefaultFolderType.INBOX);
+            } else if (UnifiedINBOXAccess.TRASH.equals(fullname)) {
+                tmp.setDefaultFolderType(DefaultFolderType.TRASH);
+            } else if (UnifiedINBOXAccess.SENT.equals(fullname)) {
+                tmp.setDefaultFolderType(DefaultFolderType.SENT);
+            } else if (UnifiedINBOXAccess.SPAM.equals(fullname)) {
+                tmp.setDefaultFolderType(DefaultFolderType.SPAM);
+            } else if (UnifiedINBOXAccess.DRAFTS.equals(fullname)) {
+                tmp.setDefaultFolderType(DefaultFolderType.DRAFTS);
+            } else {
+                tmp.setDefaultFolderType(DefaultFolderType.NONE);
+            }
             // Gather counts
             int totaCount = 0;
             int unreadCount = 0;
